// This program shows you how easy to write your own simple GA.
// All you have to do is to write your fitness function(maximize problem)
// and to write your own main function.

#include <iostream>
#include "baseenv.h"
#include "benchtime.h"
#include "gafuncs.h"

using namespace std;

/* NOTICE:
   some function has been already implemented and contained in the GAPPA
   library package.

double rastrigin(const double *x, int n)
{
	int i;
	double d;
	for(i = 0, d = 0; i < n; i++)
		d += x[i] * x[i] - 10 * cos(2 * 3.14159 * x[i]);
	return -(10 * n + d);

}
*/

main(int argc, char *argv[])
{
	// Initialize
	BaseGAEnvironment ga(rastrigin);

	// Load parameters
	if(!ga.loadSettings(argc, argv, "sample.ini"))
	{
		cout << gappa_help;
		return 0;
	}

	// termination by convergence
	bool f = clParamExists(argc, argv, "conv");
	bool fShowGene = clParamExists(argc, argv, "gene");

	HighResTimer tm;
	// Trials
	int n, gens;
	for(n = gens = 0; n < ga.getGASettings()->nTrials; n++)
	{
		ga.resetEnvironment();

		// Execute
		tm.startCounter();
		if(f)
		{
			do{
				ga.nextGeneration();
				if(fShowGene)
					cout << ga(0) << ":" << ga(0).fitness() << endl;
			}while(ga(0).fitness() != 0);
		}
		else
			ga.run();

		gens += ga.getTimer();
		cout << "." << flush;
	}

	// show result
	double t = (double)tm.getCounter() / 1000000 / ga.getGASettings()->nTrials;
	cout << " " << t
		<< "\t" << (double)gens / ga.getGASettings()->nTrials
		<< endl;

	// Show detail result
	// ga(n) is the n-th Gene instance in environment ga,
	// and ga(0) is the best Gene.
	if(clParamExists(argc, argv, "detail"))
		cout << ga(0) << ":" << ga(0).fitness() << endl;

	return 0;
}
