// This example describes how to write debugging code with GAPPA
// 1999 Programmed by T.Kawasaki
#include <iostream>
#include "baseenv.h"
#include "gafuncs.h"

using namespace std;

/* NOTICE:
   some function has been already implemented and contained in the GAPPA
   library package.

double rastrigin(const double *x, int n)
{
	int i;
	double d;
	for(i = 0, d = 0; i < n; i++)
		d += x[i] * x[i] - 10 * cos(2 * 3.14159 * x[i]);
	return -(10 * n + d);

}
*/

// sample environment
class SampleEnvironment : public BaseGAEnvironment
{
public:
	SampleEnvironment(double_fitness_func func) : BaseGAEnvironment(func) {}
};

// sample monitor : simple debugger(tracer)
class SampleDebugger : public GAMonitor
{
public:
	bool monitor(const GeneEnvironment *env, int stage, constgene_pointer *array, int popsize);
};

// sample monitor main method
// This sample is to check crossover's role in SGA,
// simply compare the average of fitness before and after crossover
// if you want more information about monitor, refer to man.txt
bool SampleDebugger::monitor(const GeneEnvironment *env, int stage, constgene_pointer *apGene, int nPop)
{
	int i;
	double avg;
	switch(stage)
	{
	case 0: // just before Migration
		break;

	case 1: // between Migration and Selection
		break;

	case 2: // between Selection and Crossover
		avg = 0;
		for(i = 0; i < nPop; i++)
			avg += apGene[i]->fitness();
		avg /= nPop;
		cout << "before crossover " << avg << endl;
		break;

	case 3: // between Crossover and Mutation
		avg = 0;
		for(i = 0; i < nPop; i++)
			avg += apGene[i]->fitness();
		avg /= nPop;
		cout << "after crossover  " << avg << endl;
		break;

	case 4: // between Mutation and Elite Processing
		break;
	}
	return false;
}

main(int argc, char *argv[])
{
	SampleEnvironment ga(rastrigin);
	if(!ga.loadSettings(argc, argv, "sample.ini"))
	{
		cout << gappa_help;
		return 0;
	}

	SampleDebugger dbg;
	ga.addObserver(&dbg);

	int trial, t;
	for(trial = 0; trial < ga.getGASettings()->nTrials; trial++)
	{
		cout << "TRIAL" << trial + 1 << " START !" << endl;
		for(t = 0; t < ga.getGASettings()->nGenerations; t++)
			ga.nextGeneration();
		cout << "Ok" << endl;
	}

	return 0;
}