// shared memory distributed GA

#include <iostream>
#include "baseenv.h"
#include "communicator.h"
#include "benchtime.h"
#include "util.h"
#include "gafuncs.h"

using namespace std;

main(int argc, char *argv[])
{
	CyclicMigration cm;
	SharedMemoryComm smc(cm, argc, argv, "sample.ini");
	if(smc.wantHelp())
	{
		cout << gappa_help;
		return 0;
	}

	const GA_GLOBAL_SETTINGS *pggs = smc.getGASettings();

	BaseGAEnvironment **envs = new BaseGAEnvironment *[pggs->nEnvironments];

	double_fitness_func func = rastrigin;
	double openMax = 5.12, closedMin = -5.12;
	if(strcmp(pggs->function_2_load, "schwefel") == 0)
		func = schwefel, openMax = 512, closedMin = -512;
	if(strcmp(pggs->function_2_load, "griewank") == 0)
		func = griewank, openMax = 512, closedMin = -512;
	if(strcmp(pggs->function_2_load, "rosenbrock") == 0)
		func = rosenbrock, openMax = 2.048, closedMin = -2.048;
	if(strcmp(pggs->function_2_load, "ridge") == 0)
		func = ridge, openMax = 64, closedMin = -64;

	int i, j;
	for(i = 0; i < pggs->nEnvironments; i++)
	{
		envs[i] = new BaseGAEnvironment(func, &smc);
		envs[i]->loadSettings(argc, argv, "sample.ini", i);
		envs[i]->setVarRange(closedMin, openMax);
	}
	smc.reset();

	fprintf(stderr,
		"number of variables : %d\n"
		"design variable     : %d [%lf %lf)\n"
		"chromo length       : %d\n"
		"population size     : %d\n"
		"number of elite     : %d\n"
		"mutation rate       : %lf\n"
		"crossover rate      : %lf\n"
		"emigration rate     : %lf\n"
		"migration interval  : %d\n"
		"number of envs      : %d\n"
		"process generations : %d\n"
		"number of trials    : %d\n",
		pggs->nVariables,
		pggs->dvd.nBit, pggs->dvd.closedMin, pggs->dvd.openMax,
		envs[0]->getChromoLengthByBit(),
		envs[0]->getPopulationSize(),
		envs[0]->getNumEliteSurvive(),
		envs[0]->getMutationRate(),
		envs[0]->getCrossoverRate(),
		envs[0]->getEmigrationRate(),
		envs[0]->getMigrationInterval(),
		pggs->nEnvironments,
		pggs->nGenerations,
		pggs->nTrials);

	HighResTimer timer;

	int trials, tm = 0;
	for(trials = 0; trials < pggs->nTrials; trials++)
	{
		for(i = 0; i < pggs->nEnvironments; i++)
			envs[i]->resetEnvironment();
		timer.startCounter();
		for(i = 0; i < pggs->nGenerations; i++)
		{
			if(i % 100 == 0) cout << "." << flush;
			for(j = 0; j < pggs->nEnvironments; j++)
			{
				envs[j]->nextGeneration();
				if(fabs(envs[j]->sortedGene(0).fitness()) <= DBL_EPSILON)
					goto ga_fin;
			}
		}
		ga_fin:;
		tm += timer.getCounter();
		if(j == pggs->nEnvironments) j = 0;
		cout << " " << i << " " << envs[j]->sortedGene(0).fitness() << endl;
	}
	cout << endl << tm / 5 << endl;

	bool f = clParamExists(argc, argv, "show");
	for(i = 0; i < pggs->nEnvironments; i++)
	{
		double sol = envs[i]->sortedGene(0).fitness();
		u32 *pn = (u32 *)&sol;
		if(f)
			cout << i << ", " << envs[i]->sortedGene(0) << ":"
				<< sol << "(" << hex << pn[0]
				<< "," << hex << pn[1] << ")" << endl;
		delete envs[i];
	}
	delete[] envs;
	return 0;
}
