//--------------------------------------------------------------------------
// benchtime.cpp
// GAPPA Parallel Distributed Environment Genetic Algorithm Library
// High Resolution Performance Timer for Benchmark Programs
//--------------------------------------------------------------------------
// Copyright (C) 2000 Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki,
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright  notice
//    and this list of conditions.
// 2. Redistributions in binary form must reproduce the above  copyright notice
//    and this list of conditions in the documentation and/or other materials
//    provided with the distribution.
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//
//    This product includes software developed by Takashi Kawasaki,
//    Tomoyuki Hiroyasu and Mitsunori Miki except the functions of sgenrand()
//    and genrand().
//
// 4. The names of Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki
//    should not be used to endorse or promote products derived from this
//    software without specific prior written permission.

#include <cstdio>
#include "benchtime.h"

using namespace std;

HighResTimer::HighResTimer()
{
#if defined(WIN32_PLATFORM)
	QueryPerformanceFrequency(&freq);
	fHighRes = freq.QuadPart;
#endif
}

void HighResTimer::startCounter()
{
#if defined(WIN32_PLATFORM)
	if(fHighRes)
		QueryPerformanceCounter(&start);
	else
		GetSystemTime(&start_st);
#else // UNIX
	struct timezone tz;
	gettimeofday(&start, &tz);
#endif
}

int HighResTimer::getCounter() const
{
#if defined(WIN32_PLATFORM)
	if(fHighRes)
	{
		LARGE_INTEGER tt;
		QueryPerformanceCounter(&tt);
		tt.QuadPart -= start.QuadPart;
		return (int)(tt.QuadPart  * 1000000 / freq.QuadPart);
	}

	SYSTEMTIME st;
	GetSystemTime(&st);
	FILETIME start_ft, ft;
	SystemTimeToFileTime(&start_st, &start_ft);
	SystemTimeToFileTime(&st, &ft);
	LARGE_INTEGER *start_li, *li;
	start_li = (LARGE_INTEGER *)&start_ft;
	li = (LARGE_INTEGER *)&ft;

	return (int)((li->QuadPart - start_li->QuadPart) / 10);

#else // UNIX
	struct timezone tz;
	struct timeval tv;
	gettimeofday(&tv, &tz);

	return (int)(tv.tv_sec * 1000000 + tv.tv_usec
		- start.tv_sec * 1000000 - start.tv_usec);
#endif
}

std::ostream& operator<<(std::ostream& out, const HighResTimer& tm)
{
	char buf[32];
	int n = tm.getCounter();
	sprintf(buf, "%d.%03d", n / 1000 / 1000, n / 1000 % 1000);
	out << buf;
	return out;
}
