//--------------------------------------------------------------------------
// FILENAME    : inetbase.h
// DESCRIPTION : base level socket manipulation library
// Programmed by T.Kawasaki
//--------------------------------------------------------------------------
#ifndef _INET_BASE_LVL_SOCKMANIP_H_
#define _INET_BASE_LVL_SOCKMANIP_H_

#if defined(_WIN32) && !defined(__CYGWIN__)
#define WINDOWS_CC
#endif

#if !defined(u32)
// 32bit, 16bit, 8bit value type : suppose normal 32bit compiler
// if your compiler is not 32bit-based, change these macros.
#define u32 unsigned int   // DWORD
#define u16 unsigned short // WORD
#define u8  unsigned char  // BYTE
#define i32 int
#define i16 short
#define i8 char
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WINDOWS_CC
// Windows
#include <windows.h>
#include <winsock.h>
#else
// maybe UNIX
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define SOCKET int
#define INVALID_SOCKET -1
#define closesocket(a) close(a)
#endif

typedef struct sockaddr_in SOCKADDR_IN;

#ifdef __cplusplus
extern "C" {
#endif

u32 get_hostaddr(const char *hostname);
u32 get_myhostaddr();
u16 split_portinfo(char *hostname);
int cleanup_socket_env();
int setup_socket_env();
int init_hostinfo(struct sockaddr_in *addr, const char *hostname, u16 port);
int init_hostinfoA(SOCKADDR_IN *addr, u32 ip, u16 port);
void get_hostinfo(char *host, u16 *pport, const SOCKADDR_IN *psi);
const char *get_hostaddrstr(u32 addr, char *buffer);

int s_write(SOCKET fd, const void *pv, int size);
int s_read(SOCKET fd, void *pv, int size);
void s_write_i32(SOCKET fd, i32 n);
void s_write_i16(SOCKET fd, i16 n);
i32 s_read_i32(SOCKET fd);
i16 s_read_i16(SOCKET fd);

int sfputs(SOCKET fd, const char *p);
int sfgets(SOCKET fd, char *buffer);

SOCKET prepare_accept(u16 port);
SOCKET s_connect(const char *hostname, u16 port, SOCKADDR_IN *paddr);
SOCKET s_connectA(u32 ip, u16 port, SOCKADDR_IN *paddr);
SOCKET s_connect_host(const char *hostname, SOCKADDR_IN *paddr);
SOCKET s_connect_sockaddr(SOCKADDR_IN *paddr);
SOCKET s_accept(SOCKET fd_accept, SOCKADDR_IN *paddr);
int s_select(SOCKET socklist[], int available[], int num, int timeout);
SOCKET accept_if_any(SOCKET fd_accept, SOCKADDR_IN *paddr);
#ifdef __cplusplus
}
#endif

#endif // _INET_BASE_LVL_SOCKMANIP_H_
