//--------------------------------------------------------------------------
// inifile.h
// GAPPA Parallel Distributed Environment Genetic Algorithm Library
// read/write windows .ini formated file
//--------------------------------------------------------------------------
// Copyright (C) 2000 Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki,
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright  notice
//    and this list of conditions.
// 2. Redistributions in binary form must reproduce the above  copyright notice
//    and this list of conditions in the documentation and/or other materials
//    provided with the distribution.
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//
//    This product includes software developed by Takashi Kawasaki,
//    Tomoyuki Hiroyasu and Mitsunori Miki except the functions of sgenrand()
//    and genrand().
//
// 4. The names of Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki
//    should not be used to endorse or promote products derived from this
//    software without specific prior written permission.

#ifndef _WIN_INIFILE_READER_H_
#define _WIN_INIFILE_READER_H_

#include <cstdlib>
#include <cmath>
#include "sysdep.h"

typedef struct tagIniFile_FieldInfo
{
	char *buffer;
	char *name;
	char *value;
	struct tagIniFile_FieldInfo *prev, *next;
}
INIFILE_FIELDINFO;

class IniFile
{
public:
	explicit IniFile(const char *filename);
	~IniFile();

	const char *readString(const char *section, const char *name, const char *def = NULL) const;
	bool readBool(const char *section, const char *name, bool def = 0) const;
	double readDouble(const char *section, const char *name, double def = 0) const;
	int readInteger(const char *section, const char *name, int def = 0) const;
	u32 readU32(const char *section, const char *name, u32 def = 0) const;

	int readExpandedString(char *buffer, const char *section, const char *name) const;
	double evaluateValue(const char *section, const char *name) const;

	void writeString(const char *section, const char *name, const char *str);
	void writeBool(const char *section, const char *name, bool f);
	void writeDouble(const char *section, const char *name, double d);
	void writeInteger(const char *section, const char *name, int n);
	void writeU32(const char *section, const char *name, u32 u);

private:
	char fn[260];
	bool fChanged;
	int count;
	INIFILE_FIELDINFO *pList;

	void write_string(const char *section, const char *name, const char *str);
	const char* expand(const char *name) const;
};

#endif // _WIN_INIFILE_READER_H_
