//--------------------------------------------------------------------------
// mpicomm.h
// GAPPA Parallel Distributed Environment Genetic Algorithm Library
// DGA MPI-Communicator Object
//--------------------------------------------------------------------------
// Copyright (C) 2000 Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki,
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright  notice
//    and this list of conditions.
// 2. Redistributions in binary form must reproduce the above  copyright notice
//    and this list of conditions in the documentation and/or other materials
//    provided with the distribution.
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//
//    This product includes software developed by Takashi Kawasaki,
//    Tomoyuki Hiroyasu and Mitsunori Miki except the functions of sgenrand()
//    and genrand().
//
// 4. The names of Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki
//    should not be used to endorse or promote products derived from this
//    software without specific prior written permission.
#ifndef _MPI_COMMUNICATOR_H_
#define _MPI_COMMUNICATOR_H_

#include <vector>
#include <mpi.h>

#if defined(_MT)
#include "muthread.h"
#endif

#include "dega.h"
#include "communicator.h"

struct ASYNC_COMM_DATA
{
	const u8 *buffer;
	MPI_Request req;
};

class MPICommunicator : public Communicator
{
	int procs, rank;
	int resetCount, sessionCount;
	int nEnvironments;
	int nEnvIdOffset;
	vector<bool> afShutdown;
	vector<GeneEnvironment *> envs;
	vector<ASYNC_COMM_DATA> isd;
	vector<int> nEnvs;
	bool fSynchronizing;
	bool fRunning;
	int argc;
	char **argv;
	char buffer_fn[256];
	char *filename;
#if defined(_MT)
	MuMutex commMutex;
#endif

	void flush_buffer();
	void check_buffer();
	void send_async(const void *buffer, int count, int type, int dest, int tag);
public:
	explicit MPICommunicator(MigrationRouter &mr, int *pargc, char ***pargv, const char *inifile, bool fSync = false);
	~MPICommunicator();
	void reset();
	void shutdown();
	int getNumEnvironments() const {return nEnvironments;}
	bool associate(GeneEnvironment *sender);
	bool disassociate(GeneEnvironment *sender);
	int getLocalEnvironmentId(GeneEnvironment *sender) const;
	int getEnvironmentId(GeneEnvironment *sender) const;

	bool migration(GeneEnvironment *sender, const constgene_pointer apEmigrants[], int nEmigrants);

	int getRank() const {return rank;}
	int getNumProcessors() const {return procs;}
	void synchronizing(bool f) {fSynchronizing = f;}

	void synchronize() const {MPI_Barrier(MPI_COMM_WORLD);}
};

#endif // _MPI_COMMUNICATOR_H_
