//--------------------------------------------------------------------------
// syncdega.cpp
// GAPPA Parallel Distributed Environment Genetic Algorithm Library
// Synchronized MPI Implementation for Win32/Pthreads
//--------------------------------------------------------------------------
// Copyright (C) 2000 Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki,
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright  notice
//    and this list of conditions.
// 2. Redistributions in binary form must reproduce the above  copyright notice
//    and this list of conditions in the documentation and/or other materials
//    provided with the distribution.
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//
//    This product includes software developed by Takashi Kawasaki,
//    Tomoyuki Hiroyasu and Mitsunori Miki except the functions of sgenrand()
//    and genrand().
//
// 4. The names of Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki
//    should not be used to endorse or promote products derived from this
//    software without specific prior written permission.

#include "syncdega.h"

syncDegaEnvironment::syncDegaEnvironment(int& argc, char** & argv) : GeneEnvironment()
{
	int i, rank, procs;

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &procs);
	setNumEnvironments(procs);
	setEnvironmentId(rank);

	// distribute seed over network
	unsigned long seed = (unsigned long)time(NULL);
	MPI_Bcast(&seed, 1, MPI_INT, 0, MPI_COMM_WORLD);

	// initializing
	randScheduler.srand(seed);
	dest = new int[procs];
	for(i = 0; i < procs; i++)
		dest[i] = (i + 1) % procs;
}

syncDegaEnvironment::~syncDegaEnvironment()
{
	delete[] dest;
	MPI_Finalize();
}

void syncDegaEnvironment::synchronize()
{
	int temp;
	MPI_Bcast(&temp, 1, MPI_INT, 0, MPI_COMM_WORLD);
}

bool syncDegaEnvironment::migrationProcedure(const constgene_pointer apEmigrants[], int nEmigrants)
{
	int rank, procs;
	rank = getEnvironmentId();
	procs = getNumEnvironments();

	int i;

	// transfer rna over network
	if(nEmigrants)
	{
		// migration scheduling
		// because all processor has same random seed,
		// all processor calcurate same schedule table, maybe...
		// this costs less than INTER-COMMUNICATION such as MPI_Bcast.
		for(i = 0; i < procs; i++)
		{
			int temp, j, k;
			j = randScheduler.genrand_int(procs);
			k = randScheduler.genrand_int(procs);
			temp = dest[j];
			dest[j] = dest[k];
			dest[k] = temp;
		}

		// sending and receiving
		int size = getRnaLength() * nEmigrants;
		u8 rna_send[size], rna_recv[size];
		MPI_Status stat;
		for(i = 0; i < nEmigrants; i++)
			apEmigrants[i]->transcription(rna_send + getRnaLength() * i);

		MPI_Sendrecv(
			rna_send, size, MPI_BYTE, dest[rank], 0,
			rna_recv, size, MPI_BYTE, MPI_ANY_SOURCE, 0,
			MPI_COMM_WORLD, &stat);

		for(i = 0; i < nEmigrants; i++)
			getVectorGeneInstance().reverseTranscription(
				rna_recv + getRnaLength() * i);
	}
	return true;
}
