//--------------------------------------------------------------------------
// sysdep.h
// GAPPA Parallel Distributed Environment Genetic Algorithm Library
// system dependency module and macros
//--------------------------------------------------------------------------
// Copyright (C) 2000 Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki,
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright	notice
//      and this list of conditions.
// 2. Redistributions in binary form must reproduce the above	copyright notice
//      and this list of conditions in the documentation and/or other materials
//      provided with the distribution.
// 3. All advertising materials mentioning features or use of this software
//      must display the following acknowledgement:
//
//      This product includes software developed by Takashi Kawasaki,
//      Tomoyuki Hiroyasu and Mitsunori Miki except the functions of sgenrand()
//      and genrand().
//
// 4. The names of Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki
//      should not be used to endorse or promote products derived from this
//      software without specific prior written permission.

#ifndef _SYSTEM_DEP_HEADER_H_
#define _SYSTEM_DEP_HEADER_H_

// insert operation if multithreaded
#if defined(_MT)
#define MT_OPERATION(op) op
#else
#define MT_OPERATION(op) /* nop */
#endif

#if defined(_WIN32) && !defined(__CYGWIN__)
#include <winsock.h>
#define WIN32_PLATFORM 1
typedef char i8;
typedef short i16;
typedef int i32;
typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;
typedef __int64 i64;
typedef unsigned __int64 u64;
#define HAVE_64BIT_TYPE 1

#define __LITTLE_ENDIAN 0xff00
#define __BIG_ENDIAN 0x00ff
#define __BYTE_ORDER __LITTLE_ENDIAN

/* this code is partially ported from ieee754.h of GNU C Library
   ***Microsoft Windows 9X/NT/2000 uses IEEE754 floting-point format,
      and there's no API to handle these values, code here provides
      handling methods only for Intel architecture, none for Alpha.

   Copyright (C) 1992, 1995, 1996, 1999 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the GNU C Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

union ieee754_double
{
	double d;

	/* This is the IEEE 754 double-precision format.	*/
	struct
	{
		unsigned int mantissa1:32;
		unsigned int mantissa0:20;
		unsigned int exponent:11;
		unsigned int negative:1;
	} ieee;

		/* This format makes it easier to see if a NaN is a signalling NaN.	*/
	struct
	{
		/* Together these comprise the mantissa.	*/
		unsigned int mantissa1:32;
		unsigned int mantissa0:19;
		unsigned int quiet_nan:1;
		unsigned int exponent:11;
		unsigned int negative:1;
	} ieee_nan;
};


#else
#include <netinet/in.h>
#define UNIX_PLATFORM 1
typedef char i8;
typedef short i16;
typedef int i32;
typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned int u32;
typedef long long i64;
typedef unsigned long long u64;
#define HAVE_64BIT_TYPE 1
#include <ieee754.h>
#endif

#if defined(_WIN32) && !defined(__CYGWIN__)
#include <windows.h>
#else
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#define GAPPA_INTERPROCLOCK_KEY 23417
#endif


class InterProcessLock
{
#if defined(_WIN32) && !defined(__CYGWIN__)
	HANDLE hMutex;
#else
	int sem_id;
#endif
public:
	explicit InterProcessLock(int n);

#if defined(_WIN32) && !defined(__CYGWIN__)
	void lock() {WaitForSingleObject(hMutex, INFINITE);}
	void unlock() {ReleaseMutex(hMutex);}
	~InterProcessLock() {CloseHandle(hMutex);}
#else
	void lock();
	void unlock();
	~InterProcessLock();
#endif

};

#endif // _SYSTEM_DEP_HEADER_H_
