//--------------------------------------------------------------------------
// util.h
// GAPPA Parallel Distributed Environment Genetic Algorithm Library
// utility module
//--------------------------------------------------------------------------
// Copyright (C) 2000 Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki,
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright  notice
//    and this list of conditions.
// 2. Redistributions in binary form must reproduce the above  copyright notice
//    and this list of conditions in the documentation and/or other materials
//    provided with the distribution.
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//
//    This product includes software developed by Takashi Kawasaki,
//    Tomoyuki Hiroyasu and Mitsunori Miki except the functions of sgenrand()
//    and genrand().
//
// 4. The names of Takashi Kawasaki, Tomoyuki Hiroyasu and Mitsunori Miki
//    should not be used to endorse or promote products derived from this
//    software without specific prior written permission.

#ifndef _DEGA_UTILITY_H_
#define _DEGA_UTILITY_H_

#include "dega.h"
#include "inifile.h"

typedef struct
{
	int nVariables;
	int nGenerations;
	int nEnvironments;
	DesignVariableDomain dvd;
	char function_2_load[260];
	char displayFormat[20];
	bool fUseGrayCode;
	int nTrials;
	bool fCheckConvergence;
}GA_GLOBAL_SETTINGS;

bool loadLocalConfiguration(GeneEnvironment *env, IniFile& ini, const char *section);

bool loadConfiguration(GA_GLOBAL_SETTINGS *pggs, GeneEnvironment *env, IniFile& ini, const char *section, bool fSetGlobalSettings = false);

bool loadConfiguration(GA_GLOBAL_SETTINGS *pggs, GeneEnvironment *env, IniFile& ini, int n, bool fSetGlobalSettings = false);

bool loadConfiguration(GA_GLOBAL_SETTINGS *pggs, GeneEnvironment *env, const char *filename, const char *section, bool fSetGlobalSettings = false);

bool loadConfiguration(GA_GLOBAL_SETTINGS *pggs, GeneEnvironment *env, const char *filename, int n, bool fSetGlobalSettings = false);

bool loadCommandlineSetttings(GA_GLOBAL_SETTINGS *pggs, GeneEnvironment *env, int argc, const char * const argv[], bool fSetGlobalSettings = true);

bool loadSettings(GA_GLOBAL_SETTINGS *pggs, GeneEnvironment *env, int argc, const char * const argv[], const char *inifile, int n = 0, bool fSetGlobalSettings = false);

bool loadGlobalSettings(GA_GLOBAL_SETTINGS *pggs, char *filename);

const char *clParam(int argc, const char * const argv[], const char *field);
bool clParamExists(int argc, const char * const argv[], const char *field);

void mpi_trace(const char *format, ...);

extern const char *gappa_help;

#endif // _DEGA_UTILITY_H_
